#!/usr/bin/env python
# coding=utf8
import tornado
from tornado.web import RedirectHandler, _unquote_or_none, ErrorHandler

from pfrock.core.constants import ROUTER_METHOD


class MyApplication(tornado.web.Application):
    def start_request(self, server_conn, request_conn):
        # Modern HTTPServer interface
        return _MyRequestDispatcher(self, request_conn)

    def __call__(self, request):
        # Legacy HTTPServer interface
        dispatcher = _MyRequestDispatcher(self, None)
        dispatcher.set_request(request)
        return dispatcher.execute()


class _MyRequestDispatcher(tornado.web._RequestDispatcher):
    def _find_handler(self):
        # Identify the handler to use as soon as we have the request.
        # Save url path arguments for later.
        app = self.application
        handlers = app._get_host_handlers(self.request)
        if not handlers:
            self.handler_class = RedirectHandler
            self.handler_kwargs = dict(url="%s://%s/" % (self.request.protocol, app.default_host))
            return
        for spec in handlers:
            match = spec.regex.match(self.request.path)
            if match:

                has_method_defined = ROUTER_METHOD in spec.kwargs
                if (not has_method_defined) or spec.kwargs.get(ROUTER_METHOD,
                                                               []) == [] or self.request.method.upper() in spec.kwargs.get(
                    ROUTER_METHOD, []):

                    self.handler_class = spec.handler_class
                    self.handler_kwargs = spec.kwargs
                    if spec.regex.groups:
                        # Pass matched groups to the handler.  Since
                        # match.groups() includes both named and
                        # unnamed groups, we want to use either groups
                        # or groupdict but not both.
                        if spec.regex.groupindex:
                            self.path_kwargs = dict(
                                (str(k), _unquote_or_none(v))
                                for (k, v) in match.groupdict().items())
                        else:
                            self.path_args = [_unquote_or_none(s)
                                              for s in match.groups()]
                    return

        if app.settings.get('default_handler_class'):
            self.handler_class = app.settings['default_handler_class']
            self.handler_kwargs = app.settings.get(
                'default_handler_args', {})
        else:
            self.handler_class = ErrorHandler
            self.handler_kwargs = dict(status_code=404)


class PfrockRequestDispatcher(tornado.web.RequestHandler):
    def initialize(self, **kwargs):
        pass



#!/usr/bin/env python
# coding=utf8
from logging.config import dictConfig


def make_logging(debug_mode=False):
    logging_level = "INFO"
    if debug_mode:
        logging_level = "DEBUG"

    logging_config = {

        'version': 1,
        'formatters': {
            'verbose': {
                'format': '[%(levelname)1.1s %(asctime)s %(name)s %(threadName)s %(module)s:%(lineno)d] %(message)s'
            },
            'simple': {
                'format': '[%(levelname)1.1s %(asctime)s %(name)s %(threadName)s %(module)s:%(lineno)d] %(message)s'
            },
        },
        'handlers': {
            'console': {
                'level': 'DEBUG',
                'class': 'logging.StreamHandler',
                'formatter': 'simple'
            }
        },
        'loggers': {
            'root': {'level': logging_level, 'handlers': ['console']},
            'tornado': {'level': logging_level, 'handlers': ['console']},
            'pfrock': {'level': logging_level, 'handlers': ['console']},
        }
    }

    dictConfig(logging_config)



# !/usr/bin/env python
# coding=utf8
import json
import traceback

from tornado.web import RequestHandler

from pfrock.cli import logger
from pfrock.core.constants import PFROCK_CONFIG_SERVER, PFROCK_CONFIG_ROUTER, PFROCK_CONFIG_PORT, ROUTER_METHOD, \
    ROUTER_PATH, ROUTER_OPTIONS, ROUTER_HANDLER
from pfrock.core.lib import auto_str


@auto_str
class PfrockConfigRouter(object):
    SUPPORTED_METHODS = RequestHandler.SUPPORTED_METHODS

    def __init__(self, path, methods, handler, options={}):
        self.path = path
        self.handler = handler
        self.options = options
        self.methods = []

        if methods == "any":
            self.methods = []
        else:
            for method in methods:
                method = method.upper()
                if method in self.SUPPORTED_METHODS:
                    self.methods.append(method)


@auto_str
class PfrockConfigServer(object):
    def __init__(self, routes, port):
        self.routes = routes
        self.port = port


class PfrockConfigParser(object):
    @classmethod
    def _parse_router(cls, router):
        path = router[ROUTER_PATH] if ROUTER_PATH in router else None
        methods = router[ROUTER_METHOD] if ROUTER_METHOD in router else []
        handler = router[ROUTER_HANDLER] if ROUTER_HANDLER in router else None
        options = router[ROUTER_OPTIONS] if ROUTER_OPTIONS in router else None
        if path and handler:
            return PfrockConfigRouter(path, methods, handler, options)
        return None

    @classmethod
    def _parse_routers(cls, routers):
        router_list = []
        for router in routers:
            router = cls._parse_router(router)
            if router:
                router_list.append(router)
        return router_list

    @classmethod
    def _parse_servers(cls, server):
        port = server[PFROCK_CONFIG_PORT] if PFROCK_CONFIG_PORT in server else None
        routers = cls._parse_routers(server[PFROCK_CONFIG_ROUTER]) if PFROCK_CONFIG_ROUTER in server else None
        if port and routers:
            return PfrockConfigServer(routers, port)

    @classmethod
    def do(cls, config_file_path):
        with open(config_file_path, 'r') as fin:

            try:
                config_data = json.load(fin)
            except:
                logger.error("%s not well formed \n%s" % (config_file_path, traceback.format_exc()))
                return None

            config_servers = config_data[PFROCK_CONFIG_SERVER] if PFROCK_CONFIG_SERVER in config_data else None
            if config_servers:
                for config_server in config_servers:
                    config_server = cls._parse_servers(config_server)
                    # todo: dev version just support one server
                    return config_server
            return None



#!/usr/bin/env python
# coding=utf8
import importlib


def auto_str(cls):
    def __str__(self):
        return '%s(%s)' % (
            type(self).__name__,
            ', '.join('%s=%s' % item for item in vars(self).items())
        )

    cls.__str__ = __str__
    return cls


def import_from_file(class_path):
    r_index = class_path.rfind('.')
    module = importlib.import_module(class_path[0:r_index])
    my_class = getattr(module, class_path[r_index + 1:])
    return my_class



#!/usr/bin/env python
# coding=utf8
from pfrock.core import logger
from pfrock.core.plugin import PLUGIN_CLASS_KEY_REGISTER, PLUGIN_CLASS_GET_HANDLER


class PfrockPluginRegister(object):
    register_class_plugins = {}

    @classmethod
    def register(cls, module_class, module_name):

        if hasattr(module_class, PLUGIN_CLASS_KEY_REGISTER):

            handler_class = getattr(module_class, PLUGIN_CLASS_KEY_REGISTER)
            invert_op = getattr(handler_class, PLUGIN_CLASS_GET_HANDLER, None)
            if callable(invert_op):
                cls.register_class_plugins[module_name] = handler_class
                # logger.debug("register plugin %s " % handler_class)
                return
            else:
                logger.warn("cannot find %s in %s " % (PLUGIN_CLASS_GET_HANDLER, handler_class))
                return

        logger.warn("cannot find %s in %s " % (PLUGIN_CLASS_KEY_REGISTER, module_class))

    @classmethod
    def get_handler(cls, module_name):
        return cls.register_class_plugins[module_name] if module_name in cls.register_class_plugins else None



#!/usr/bin/env python
# coding=utf8
import importlib
import traceback

from pfrock.core import logger
from pfrock.core.constants import ROUTER_METHOD
from pfrock.core.plugin import PLUGIN_CLASS_GET_HANDLER
from pfrock.core.register import PfrockPluginRegister


class RoutesMgr(object):
    @classmethod
    def _import_route(cls, module_name):

        # import
        try:
            module_class = importlib.import_module(module_name)

            PfrockPluginRegister.register(module_class, module_name)
            return True
        except:
            logger.warn("cannot import plugin handler %s \n %s" % (module_name, traceback.format_exc()))
            return False

    @classmethod
    def _get_http_handler(cls, one_route, register_handler_class):

        try:

            handlers = getattr(register_handler_class(),
                               PLUGIN_CLASS_GET_HANDLER)(one_route.options,
                                                         path=one_route.path,
                                                         methods=one_route.methods)

            real_handlers = []
            if handlers:
                handlers = handlers if type(handlers) == list else [handlers]
                # append methods for handler options
                for handler in handlers:
                    if type(handler) == tuple:
                        real_handler = handler
                    else:
                        real_handler = (one_route.path, handler, one_route.options)

                    real_handler[2][ROUTER_METHOD] = one_route.methods
                    real_handlers.append(real_handler)
                return real_handlers

            return []

        except:
            logger.warn(traceback.format_exc())
            return []

    @classmethod
    def _get_one_route(cls, one_route):

        register_handler_class = PfrockPluginRegister.get_handler(one_route.handler)
        if register_handler_class:
            return cls._get_http_handler(one_route, register_handler_class)
        else:
            return None

    @classmethod
    def get_routes(cls, config_server):
        routes = config_server.routes

        route_list = []
        for route in routes:

            def add_route(route):
                if route:
                    logger.debug("add : " + str(route))
                    route_list.append(route)

            if cls._import_route(route.handler):
                cur_routes = cls._get_one_route(route)
                if type(cur_routes) == list:
                    for cur_route in cur_routes:
                        add_route(cur_route)
                else:
                    add_route(cur_routes)

        return route_list



#!/usr/bin/env python
# coding=utf8
import os

import pfrock


def print_logo():
    logo_path = os.path.join(os.path.dirname(pfrock.__file__), "logo.txt")
    try:
        with open(logo_path, 'r') as fin:
            print fin.read()
    except:
        pass



#!/usr/bin/env python
# coding=utf8

import os
import sys
import time

import pfrock
from pfrock.cli import argument_parser
from pfrock.cli.config_parser import PfrockConfigParser
from pfrock.cli.log import make_logging
from pfrock.cli.logo import print_logo
from pfrock.core import PFrock
from pfrock.core.routes import RoutesMgr

pfrockfile = 'pfrockfile.json'


def main():
    sys.path.insert(0, os.getcwd())

    # argument parser
    debug, no_watch = argument_parser()

    # log
    make_logging(debug)
    print_logo()
    print "pfrock version %s " % pfrock.__version__
    time.sleep(1)

    # parser
    config_server = PfrockConfigParser.do(pfrockfile)

    # routes
    if config_server:
        route_list = RoutesMgr.get_routes(config_server)
        # new frock
        p_frock = PFrock(auto_reload=not no_watch, port=config_server.port)
    else:
        route_list = []
        # new frock
        p_frock = PFrock(auto_reload=not no_watch)

    p_frock.add_watch(pfrockfile)
    p_frock.add_handler(route_list)

    # start
    try:
        p_frock.start()
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# coding=utf8
import logging

import tornado.autoreload
import tornado.ioloop
import tornado.web

from pfrock.core.web import MyApplication

logger = logging.getLogger('pfrock.core')


class PFrock(object):
    def __init__(self, auto_reload=True, port=8888):
        self.app = MyApplication(autoreload=True)
        self.auto_reload = auto_reload
        self.port = port
        logger.info("started server " + str(port) + (" with autoreload mode" if self.auto_reload else ""))

    def add_handler(self, handlers):
        self.app.add_handlers(".*$", handlers)

    def start(self):
        self.app.listen(self.port)
        tornado.ioloop.IOLoop.current().start()

    def add_watch(self, watch_file):
        tornado.autoreload.watch(watch_file)



#!/usr/bin/env python
# coding=utf8

import argparse
import logging

logger = logging.getLogger('pfrock.cli')


def argument_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--debug", action="store_true", dest='debug',
                        help="Enable verbose (debug) logging. Default do not output debug info.")
    parser.add_argument("-w", "--nowatch ", action="store_true", dest='nowatch',
                        help="Don't watch the pfrockfile for changes. Default is watching.")
    args = parser.parse_args()
    return args.debug, args.nowatch



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import io
import os
import re

from pip.req import parse_requirements
from setuptools import setup, find_packages

f = open(os.path.join(os.path.dirname(__file__), 'README.rst'))
readme = f.read()
f.close()

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements('requirements.txt', session=False)
# reqs is a list of requirement
# e.g. ['django==1.5.1', 'mezzanine==1.4.6']
reqs = [str(ir.req) for ir in install_reqs]


def read(*names, **kwargs):
    return io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ).read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
    name='pfrock',
    version=find_version("pfrock/__init__.py"),
    description='A plugin-based tool for running fake HTTP and socket services using Python.',
    long_description=readme,
    author='knightliao',
    author_email='knightliao@gmail.com',
    url='https://github.com/knightliao/pfrock',
    platforms='any',
    license="http://www.apache.org/licenses/LICENSE-2.0",
    packages=find_packages(exclude=['tests', 'tests.*']),
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    package_data={
        'pfrock': ['logo.txt'],
    },
    install_requires=reqs,
    scripts=[],
    entry_points={
        'console_scripts': [
            'pfrockpy = pfrock.console_scripts:main',
        ],
    }

)



#!/usr/bin/env python
# coding=utf8

# router
ROUTER = 'routes'
ROUTER_PATH = "path"
ROUTER_METHOD = "methods"
ROUTER_HANDLER = "handler"
ROUTER_OPTIONS = "options"

# pfrock
PFROCK_CONFIG_SERVER = 'servers'
PFROCK_CONFIG_ROUTER = 'routes'
PFROCK_CONFIG_PORT = 'port'
